"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const supertest_1 = __importDefault(require("supertest"));
const Yup = __importStar(require("yup"));
const express_yup_middleware_1 = require("../src/express-yup-middleware");
const createAppWithPath = ({ path, middleware }) => {
    const app = express_1.default();
    app.use(path, middleware);
    return app;
};
describe('express yup middleware', () => {
    let agent = null;
    it('creates an express middleware', () => {
        const expressMiddleware = express_yup_middleware_1.expressYupMiddleware({ schemaValidator: { schema: {} } });
        expect(typeof expressMiddleware).toBe('function');
    });
    describe('when validating the request query, body and params properties', () => {
        describe('without custom error messages', () => {
            const schemaValidator = {
                schema: {
                    query: {
                        yupSchema: Yup.object().shape({
                            testQueryParam: Yup.string().required('requiredTestQueryParam'),
                        }),
                    },
                    body: {
                        yupSchema: Yup.object().shape({
                            testBodyProperty: Yup.string().required('requiredTestBodyProperty'),
                        }),
                    },
                    params: {
                        yupSchema: Yup.object().shape({
                            testParams: Yup.string().required('requiredTestParams'),
                        }),
                    },
                },
            };
            beforeEach(() => {
                const app = createAppWithPath({
                    path: '/test',
                    middleware: express_yup_middleware_1.expressYupMiddleware({ schemaValidator }),
                });
                agent = supertest_1.default(app);
            });
            it('returns a bad request error showing yup validation messages', () => __awaiter(void 0, void 0, void 0, function* () {
                const { body } = yield agent.get('/test').expect(400);
                expect(body).toStrictEqual({
                    errors: {
                        query: [
                            {
                                message: 'requiredTestQueryParam',
                                propertyPath: 'testQueryParam',
                            },
                        ],
                        body: [
                            {
                                message: 'requiredTestBodyProperty',
                                propertyPath: 'testBodyProperty',
                            },
                        ],
                        params: [
                            {
                                message: 'requiredTestParams',
                                propertyPath: 'testParams',
                            },
                        ],
                    },
                });
            }));
        });
        describe('using custom error messages', () => {
            const schemaValidator = {
                schema: {
                    body: {
                        yupSchema: Yup.object().shape({
                            testBodyProperty: Yup.string().required('requiredTestBodyProperty'),
                        }),
                    },
                },
                errorMessages: {
                    requiredTestBodyProperty: {
                        key: 'tes-body-property-required',
                        message: 'The "testBodyProperty" property is required!',
                    },
                },
            };
            beforeEach(() => {
                const app = createAppWithPath({
                    path: '/test',
                    middleware: express_yup_middleware_1.expressYupMiddleware({ schemaValidator }),
                });
                agent = supertest_1.default(app);
            });
            it('returns a bad request error showing yup validation messages', () => __awaiter(void 0, void 0, void 0, function* () {
                const { body } = yield agent.get('/test').expect(400);
                expect(body).toStrictEqual({
                    errors: {
                        body: [
                            Object.assign(Object.assign({}, schemaValidator.errorMessages.requiredTestBodyProperty), { propertyPath: 'testBodyProperty' }),
                        ],
                    },
                });
            }));
        });
    });
    describe('when validating a custom request property', () => {
        const schemaValidator = {
            schema: {
                headers: {
                    yupSchema: Yup.object().shape({
                        testHeaderProperty: Yup.string().required('requiredHeaderProperty'),
                    }),
                },
            },
        };
        beforeEach(() => {
            const app = createAppWithPath({
                path: '/test',
                middleware: express_yup_middleware_1.expressYupMiddleware({ schemaValidator, propertiesToValidate: ['headers'] }),
            });
            agent = supertest_1.default(app);
        });
        it('returns a bad request error showing yup validation messages', () => __awaiter(void 0, void 0, void 0, function* () {
            const { body } = yield agent.get('/test').expect(400);
            expect(body).toStrictEqual({
                errors: {
                    headers: [
                        {
                            message: 'requiredHeaderProperty',
                            propertyPath: 'testHeaderProperty',
                        },
                    ],
                },
            });
        }));
    });
    describe('when returning a custom validation code', () => {
        const schemaValidator = {
            schema: {
                body: {
                    yupSchema: Yup.object().shape({
                        testProperty: Yup.string().required('required'),
                    }),
                },
            },
        };
        beforeEach(() => {
            const app = createAppWithPath({
                path: '/test',
                middleware: express_yup_middleware_1.expressYupMiddleware({ schemaValidator, expectedStatusCode: 418 }),
            });
            agent = supertest_1.default(app);
        });
        it(`returns "I'm a teapot" code 418`, () => __awaiter(void 0, void 0, void 0, function* () {
            const { body } = yield agent.get('/test').expect(418);
            expect(body).toStrictEqual({
                errors: {
                    body: [
                        {
                            message: 'required',
                            propertyPath: 'testProperty',
                        },
                    ],
                },
            });
        }));
    });
});
//# sourceMappingURL=yup-middleware.spec.js.map